// QRCode Blickfang 2012, Stuttgart
// Moritz Kemper, IAD Physical Computing Lab
// moritz.kemper@zhdk.ch
// ZHdK, 08/02/2012

/* Issues:
- Fixing the Layout inside InformationOverlay.draw()
- How to stick the width of an Image to the width of the display?
- Delay sound plays > picture appears
- LED
*/

//For reading QRCodes
import com.aos.zxing4processing.*;
ZXING4P zxing;
import hypermedia.video.*;
OpenCV videoCap; 
String currentTag;
//Sound Library
import ddf.minim.*;
Minim minim;
AudioSample peep;
//BackrgoundImage
PImage backgroundImage;
//XML Reader
XMLElement xml;
//Empty Overlay Array
InformationOverlay[] overlayBox;
//How Many are we gonna draw?
int amountOfOverlays;
//How long should they stay up?
int blendOutTime = 5000;

void setup() 
{
  //Set the size to fullscreen
  size(screen.width, screen.height);

  //Capture QR Code
  videoCap = new OpenCV(this);
  videoCap.capture(640, 480);  
  zxing = new ZXING4P(this);

  //Start Sound
  minim = new Minim(this);
  peep = minim.loadSample("peep.mp3", 2048);

  //Load Backgound Image
  backgroundImage = loadImage("map.jpg");

  //Load XML File
  xml = new XMLElement (this, "content.xml");
  
  //Set Display Time 
  XMLElement[] xmlDuration = xml.getChildren("duration");
  blendOutTime = int(xmlDuration[0].getContent());
  
  //Load Images from XML
  XMLElement[] xmlName = xml.getChildren("overlay/name");
  XMLElement[] xmlImage = xml.getChildren("overlay/image");
  XMLElement[] xmlPositionX = xml.getChildren("overlay/positionX");
  XMLElement[] xmlPositionY = xml.getChildren("overlay/positionY");
  //Print the amount of Images/Overlays
  amountOfOverlays = xmlName.length;
  println(amountOfOverlays);

  //Create Overlays
  overlayBox = new InformationOverlay[xml.getChildCount()];
  for (int i=0; i < xmlName.length; i++) 
  { 
    overlayBox[i] = new InformationOverlay(xmlName[i].getContent(), xmlImage[i].getContent(), xmlPositionX[i].getContent(), xmlPositionY[i].getContent());
  }
}

void draw()
{
  //Drawing the Background
  imageMode(CENTER);
  image(backgroundImage, width/2, height/2);

  //Read the current Tag
  currentTag = ReadQRCode(); 
  if (!currentTag.equals(""))
  { 
    //Print the Tag
    println(currentTag);
  }  
  //Display the Tags
  for (int i=0; i<amountOfOverlays; i++)
  {
    //Check wether the Tag is already visible
    if (currentTag.equals(overlayBox[i].overlayName) && overlayBox[i].visible == false)
    {
      peep.trigger();
      overlayBox[i].visible = true;
      overlayBox[i].timestamp = millis();
    }
    //Check if the timestamp is still valid
    if (overlayBox[i].visible && (millis()-overlayBox[i].timestamp)<blendOutTime)
    {
      //Draw the Overlay
      overlayBox[i].draw();
    }
    else
    {
      //Un-Draw the Overlay 
      overlayBox[i].visible = false;
    }
  }
}

void stop()
{
  peep.close();
  minim.stop();
  super.stop();
}

